# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# zprtap.py
#
# création et démarrage des services zephir_web
#
###########################################################################

# filtrage des warnings à l'importation de nevow
import warnings
try:
    from twisted.python.components import ComponentsDeprecationWarning
    warnings.filterwarnings('ignore', category=ComponentsDeprecationWarning, message="Please don't use __adapt__ on Interface subclasses")
except:
    # twisted 1 : pas besoin de filtrer ce warning
    pass

from twisted.application import internet
from twisted.web import server, static
from zephir.certificats import ServerContextFactory

from zephir.web.config import IMAGE_DIR, PORT_WEB, PATH_TEMP, HELP_DIR, SESSION_TIMEOUT
from zephir.web.html import index

# options ligne de commande
from twisted.python import usage

import os, time

class Options(usage.Options):
    optParameters = [["port", "p", PORT_WEB,
                      "Numero de port pour le serveur web."]]

NOBODY_GID = NOBODY_UID = 65534

class SessionZephir(server.Session):
    """Classe de session avec timeout personnalisable
    """
    def checkExpired(self):
        """Is it time for me to expire?
        If I haven't been touched in XXX minutes, I will call my
        expire method.
        """
        if time.time() - self.lastModified > SESSION_TIMEOUT:
            if self.site.sessions.has_key(self.uid):
                self.expire()


class ZephirSite(server.Site):

    def _drop_privileges(self):
        #drop de privileges vers nobody/nobody
        #print "dropping privileges (%s)" % self.__module__.split('.')[-1]
        os.setegid(NOBODY_GID)
        os.seteuid(NOBODY_UID)

    def _gain_privileges(self):
        #on reprend nos privileges initiaux
        #print "gaining privileges (%s)" % self.__module__.split('.')[-1]
        os.seteuid(self.UID)
        os.setegid(self.GID)

    def startFactory(self):
        #sauvegarde de notre UIG/GID courant
        self.UID = os.getuid()
        self.GID = os.getgid()
        self._drop_privileges()
        # création du répertoire temporaire si besoin (supprimé en cas de reboot)
        if not os.path.isdir(PATH_TEMP):
            os.makedirs(PATH_TEMP)
        server.Site.startFactory(self)

    def stopFactory(self):
        # on reprend les privilèges pour pouvoir supprimer le fichier de PID
        self._gain_privileges()
        server.Site.stopFactory(self)

    def makeSession(self):
        """Generate a new Session instance, and store it for future reference.
        """
        uid = self._mkuid()
        session = self.sessions[uid] = SessionZephir(self, uid)
        # toutes les 15 minutes, on regarde si le délai d'inactivité
        # de la session n'est pas dépassé
        session.checkExpiredLoop.start(900)
        return session

def makeService(config):
    # port TCP à écouter
    port = int(config["port"])

    root = index.Index()
    root.putChild("images",static.File(IMAGE_DIR))
    root.putChild("aide",static.File(HELP_DIR))
    root.putChild("tmp", static.File(PATH_TEMP))
    site = ZephirSite(root)

    return internet.SSLServer(port, site, ServerContextFactory())
